local M = {}

--The hirochi_raceway map already has the textures built-in manually from this mod, and is
--used to copy as necessary
--TODO recusive function to find all directories in /gravitationalRacing
local folderPaths = {
  "/levels/hirochi_raceway/art/gravitationalRacing/celestialbodies",
  "/levels/hirochi_raceway/art/gravitationalRacing/field/spheres",
  "/levels/hirochi_raceway/art/gravitationalRacing/signs",
  "/levels/hirochi_raceway/art/gravitationalRacing/track",
  "/levels/hirochi_raceway/art/gravitationalRacing/holograms"
}

local function replaceInFile(filePath, old, new)
  --[[
  Opens a file and replaces a word with another
  ]]--
  local file = io.open(filePath, "r")
  local content = file:read("*all")
  file:close()

  content = content:gsub(old, new)

  file = io.open(filePath, "w")
  file:write(content)
  file:close()
end

local function createTextureMappings(newMap)
  --[[
  Creates the texture mappings for this map
  NOTE: it is memory efficient to not clone .png files but rename the materials.cs files (since they are ~23MB)
  and as it is fairly non-time complex, it doesn't affect performance much
  Also, most celestials do not have modified texures (ie. aren't emissive, transparent etc.) so
  do not need to be copied as they are automatically mapped on creation - again, saves memory by 10x
  ]]--
  for _, folderPath in ipairs(folderPaths) do
    local files = FS:findFilesByPattern(folderPath, "*", 1, true, false)

    for _, fileName in ipairs(files) do
      local newFileName = fileName:gsub("hirochi_raceway", newMap)

      FS:copyFile(fileName, newFileName)

      if fileName:find("materials.") then
        replaceInFile(newFileName, "hirochi_raceway", newMap)
      end
    end
  end
end

local function hasTexturesFor(map)
  --[[
  Returns whether or not a map has textures for this mod
  ]]
  if not map then
    return false
  end

  return #FS:findFilesByPattern("/levels/"..map.."/art/gravitationalRacing", "*", 1, true, false) > 0
end

local function createVanillaMapTextures()
  --[[
  A one-time use function that creates the texture mappings for the vanilla maps
  Algorithm uses hirochi_raceway as the host for copying files
  ]]--
  local vanillaMaps = {
    "automation_test_track",
    "Cliff",
    "derby",
    "driver_training",
    "east_coast_usa",
    "glow_city",
    "GridMap",
    "industrial",
    "italy",
    "jungle_rock_island",
    "port",
    "small_island",
    "Utah",
    "west_coast_usa"
  }

  for _, mapName in ipairs(vanillaMaps) do
    if not hasTexturesFor(mapName) then
      createTextureMappings(mapName)
    end
  end
end

M.createTextureMappings = createTextureMappings
M.hasTexturesFor = hasTexturesFor
M.createVanillaMapTextures = createVanillaMapTextures
return M
