angular.module('beamng.apps')

.constant("Options", {
  celestials: {
    planets: [
      {txt: "Mercury"     , val: "mercury"},
      {txt: "Venus"       , val: "venus"  },
      {txt: "Earth"       , val: "earth"  },
      {txt: "Mars"        , val: "mars"   },
      {txt: "Jupiter"     , val: "jupiter"},
      {txt: "Saturn"      , val: "saturn" },
      {txt: "Uranus"      , val: "uranus" },
      {txt: "Neptune"     , val: "neptune"}
    ],
    stars: [
      {txt: "Sun"              , val: "sun"            },
      {txt: "Neutron Star"     , val: "neutronStar"    },
      {txt: "AB Doradus A"     , val: "ABDoradusA"     },
      {txt: "Alpha Volantis"   , val: "alphaVolantis"  },
      {txt: "CD Crucis"        , val: "CDCrucis"       },
      {txt: "CHXR 73"          , val: "CHXR73"         },
      {txt: "Eta Ursae Majoris", val: "etaUrsaeMajoris"},
      {txt: "HD 49798"         , val: "HD49798"        },
      {txt: "Procyon A"        , val: "procyonA"       }
    ],
    other: [
      {txt: "Black Hole"  , val: "blackhole"}
    ]
  },

  positionTypes: [
    {txt: "Relative to Vehicle", val: "relative"},
    {txt: "Absolute"           , val: "absolute"}
  ],

  objectTypes: [
    {txt: "Static", val: "static"      },
    {txt: "Moving", val: "fixedDynamic"},
  ],

  pathTypes: [
    {txt: "Circular Orbit", val: "circular"},
    {txt: "Line"          , val: "linear"  }
  ],

  planes: [
    {txt: "X", val: "x"},
    {txt: "Y", val: "y"}
  ],

  collisions: [
    {txt: "Ignore", val: "none"},
    {txt: "Reset to Spawn", val: "respawn"}
  ],

  classes: [
    {txt: "Standard" , val: "stable"  },
    {txt: "Exotic"   , val: "exotic"  },
    {txt: "Unstable" , val: "unstable"},
    {txt: "Binary"   , val: "binary"  }
  ]
})

.directive('celestialCreator', ['Options', 'bngApi', function (Options, bngApi, gamepadNav) {
  return {
    templateUrl: 'modules/apps/CelestialCreator/templateBMNG.html',
    replace: false,
    restrict: 'E',
    scope: false,
    controller: function ($scope, $sce, $element, $attrs) {
      $scope.celestial = {value: null, options: Options.celestials};
      $scope.class = {value: null, options: Options.classes};
      $scope.position = {type: {value: null, options: Options.positionTypes}, coords: {x: 0, y: -650, z: 60}};
      $scope.radius = 50;
      $scope.passive = false;
      $scope.objectType = {value: null, options: Options.objectTypes};
      $scope.path = {
        type: {
          value: null,
          options: Options.pathTypes
        },
        settings: {
          length: 20,
          frequency: null,
          axis: {
            value: null,
            options: Options.planes
          },
          orbitingBody: "",
          orbitRadius: null
        }
      };
      $scope.availableParentCelestials = [];
      $scope.celestialsInWorld = [];
      $scope.msg = {txt: "", accept: "", decline: ""};
      $scope.deleting = "";
      $scope.settings = {acceleration: {value: 200, pending: 200}, collide: {value: null, pending: null, options: Options.collisions}};
      $scope.isPreviewing = false;
      $scope.isShowingCelestialsInWorld = false;
      $scope.overlayLayers = [];

      $scope.init = function() {
        // Set default values for those that self-reference
        $scope.celestial.value     = $scope.celestial.options.other[0].val;
        $scope.class.value         = $scope.class.options[0].val;
        $scope.position.type.value = $scope.position.type.options[0].val;
        $scope.objectType.value    = $scope.objectType.options[0].val;

        $scope.path.type.value          = $scope.path.type.options[0].val;
        $scope.path.settings.axis.value = $scope.path.settings.axis.options[0].val;

        $scope.settings.collide.value   = $scope.settings.collide.options[0].val;
        $scope.settings.collide.pending = $scope.settings.collide.options[0].val;

        // Clone
        var allCelestials = JSON.parse(JSON.stringify($scope.celestial.options));
        var other   = allCelestials.other.slice(0);
        var stars   = allCelestials.stars.slice(0);
        var planets = allCelestials.planets.slice(0);
        var arr     = [];
        $scope.celestial.options = arr.concat(planets).concat(stars).concat(other);

        bngApi.engineLua("extensions.celestialController.initialise()", (response) => {
          var state = response[0];
          if (state !== "Success") {
            $scope.showMessage(response[1], state, null, null, true);
          }
        });
      };

      $scope.toggleAxis = function() {
        bngApi.engineLua("extensions.celestialController.toggleAxis()");
      };

      $scope.toggleTOD = function() {
        bngApi.engineLua("extensions.celestialController.toggleTOD()");
      };

      $scope.formatData = function() {
        var valid = true;

        var celestialData = {
          celestial:  $scope.celestial.value,
          class:      $scope.class.value,
          position:   {
            type: $scope.position.type.value,
            coords: {
              x: $scope.position.coords.x,
              y: $scope.position.coords.y,
              z: $scope.position.coords.z
            }
          },
          radius:     $scope.radius,
          passive:    $scope.passive,
          objectType: $scope.objectType.value
        };

        // Check that the radius is defined and >0
        if ($scope.checkForError(!celestialData.radius || celestialData.radius <= 0, "label_radius")) {
          valid = false;
        }

        if (celestialData.objectType === "fixedDynamic") {
          celestialData.path = {
            type:      $scope.path.type.value,
            frequency: $scope.path.settings.frequency
          };

          var isCircular = celestialData.path.type === "circular";
          if (celestialData.class || isCircular) {
            celestialData.path.orbitRadius  = $scope.path.settings.orbitRadius;

            if (isCircular) {
              celestialData.path.orbitingBody = $scope.path.settings.orbitingBody;

              // Check that orbitingBody has been defined
              if ($scope.checkForError(!celestialData.path.orbitingBody, "label_orbitBody")) {
                valid = false;
              }
            }
          } else if (celestialData.path.type === "linear") {
            celestialData.path.length = $scope.path.settings.length;
            celestialData.path.axis   = $scope.path.settings.axis.value;

            // Check that length has been defined (naturally handles <0 since minus sign returns undefined)
            if ($scope.checkForError(!celestialData.path.length, "label_length")) {
              valid = false;
            }

            // Check that frequneucy has been defined as it cannot be calculated
            if ($scope.checkForError(!celestialData.path.frequency, "label_frequency")) {
              valid = false;
            }
          }
        }

        // Do not send incorrect data
        if (!valid) {
          return;
        }

        return celestialData;
      };

      $scope.getSelectColour = function(selected) {
        if (selected) return "#ffff00";
        else          return "";
      }

      $scope.updatePreview = function() {
        document.getElementById("preview").style.backgroundColor = $scope.getSelectColour($scope.isPreviewing);

        var data = $scope.formatData();
        if (data) {
          bngApi.engineLua("extensions.celestialController.showPreview(" + $scope.isPreviewing + ", " + bngApi.serializeToLua(data) + ")");
        }
      };

      $scope.togglePreview = function() {
        $scope.isPreviewing = !$scope.isPreviewing;
        $scope.updatePreview();
      };

      $scope.submitToLua = function() {
        var data = $scope.formatData();
        if (data) {
          bngApi.engineLua("extensions.celestialController.requestCreation(" + bngApi.serializeToLua(data) + ")", (response) => {
            var state = response[0];
            if (state !== "Success") {
              $scope.showMessage(response[1], state, null, null, true);
            } else {
              // Resets them back to black colour ie. not error red
              $scope.checkForError(false, "label_length");
              $scope.checkForError(false, "label_orbitBody");
              $scope.checkForError(false, "label_radius");

              // Removes the preview
              $scope.isPreviewing = false;
              $scope.updatePreview();

              var type = response[1];
              var celestialsCreated = response[2];
              var isOrbitable = type === "star" || type === "blackhole";

              for (var i = 0; i < celestialsCreated.length; i++) {
                var name = celestialsCreated[i];

                // Only stars and black holes can be orbited
                if (isOrbitable) {
                  $scope.availableParentCelestials.push(name);
                }

                // Since this does not trigger a $digest() call
                $scope.$apply(function() {
                  $scope.celestialsInWorld.push(name);
                });
              }
            }
          });
        }
      };

      /**
      *Removes the top overlay
      *@returns {int} - z-index of 0 as this overlay is destroyed
      */
      $scope.removeOverlay = function() {
        var id = $scope.overlayLayers.length-1;
        document.getElementById("overlay_" + id).remove();
        $scope.overlayLayers.pop();
        return 0;
      };

      /**
      *Adds in a new layer on top
      *@returns {int} - the z-index for new elements to be visible
      */
      $scope.addNewOverlay = function() {
        var id = $scope.overlayLayers.length;
        var zIndex = id + 1;

        var overlay = document.createElement("div");
        overlay.className = "overlay";
        overlay.id = "overlay_" + id;
        overlay.style.zIndex = zIndex;
        document.getElementById("ccContainer").appendChild(overlay);

        $scope.overlayLayers.push(true);

        // +1 since this layer is at z-index = zIndex
        return zIndex + 1;
      };

      /**
      *Adds or removes an overlay
      *@param {boolean} add - whether or not to add another layer
      *@param {int} - the z-index of the layer
      */
      $scope.overlayControl = function(add) {
        return add ? $scope.addNewOverlay() : $scope.removeOverlay();
      };

      $scope.toggleShowCelestialsInWorld = function() {
        $scope.isShowingCelestialsInWorld = !$scope.isShowingCelestialsInWorld;

        var zIndex = $scope.overlayControl($scope.isShowingCelestialsInWorld);
        var state = $scope.isShowingCelestialsInWorld ? "block" : "none";

        var elem = document.getElementById("edit");
        elem.style.display = state;
        elem.style.zIndex = zIndex;

        var buttonElem = document.getElementById("showCelestials");
        buttonElem.style.backgroundColor = $scope.getSelectColour($scope.isShowingCelestialsInWorld);
        buttonElem.style.zIndex = zIndex;
      };

      $scope.showMessage = function(msg, state, acceptMsg, declineMsg, generic) {
        $scope.$apply(function() {
          $scope.msg.txt = msg;

          var zIndex = $scope.overlayControl(true);

          var msgElement = document.getElementById("msgStatus");
          msgElement.src = "modules/apps/CelestialCreator/" + state.toLowerCase() + ".png";
          msgElement.style.zIndex = zIndex;

          var editElement = document.getElementById("editMsg");
          editElement.style.display = "block";
          editElement.style.zIndex = zIndex;

          var acceptElement = document.getElementById("accept");
          if (generic) {
            // Remove one of the button (accept removed as explained below vv)
            acceptElement.style.display = "none";
            acceptElement.style.zIndex = 0;
            // This only shows the decline button (since the accept button has a function attached to it already)
            $scope.msg.decline = "OK";
          } else {
            // Make sure that the delete button is there
            acceptElement.style.display = "block";

            $scope.msg.accept = acceptMsg;
            $scope.msg.decline = declineMsg;

            acceptElement.style.zIndex = zIndex;
            document.getElementById("decline").style.zIndex = zIndex;
          }
        });
      };

      $scope.applySettings = function() {
        // Values inputted
        var accelPend   = $scope.settings.acceleration.pending;
        var collidePend = $scope.settings.collide     .pending;

        // Check that acceleration has been defined (naturally handles <0 since minus sign returns undefined)
        if ($scope.checkForError(!accelPend, "label_stgs_accel")) {
          return;
        }

        // Apply the inputted values as they are acceptable
        $scope.settings.acceleration.value = accelPend  ;
        $scope.settings.collide.value      = collidePend;

        var settings = {
          acceleration: accelPend,
          collide:      collidePend
        };

        bngApi.engineLua("extensions.celestialController.applySettings(" + bngApi.serializeToLua(settings) + ")");
      };

      /**
      *Toggles the settings menu open or closed
      *@param {boolean} open - whether the menu is to be opened or closed
      */
      $scope.toggleSettings = function(open) {
        var zIndex = $scope.overlayControl(open);

        var settingsElem = document.getElementById("settings");
        settingsElem.style.zIndex = zIndex;
        settingsElem.style.display = open ? "block" : "none";
      };

      /**
      *Changes the colour of an element based on the error condition:
      * Red if true, white if false
      *@param {boolean} condition - the error condition
      *@param {Object} element - the element to change colour
      @returns {boolean} - whether there was an error
      */
      $scope.checkForError = function(condition, element) {
        document.getElementById(element).style.color = condition ? "red" : "white"
        return condition;
      };

      $scope.resetMessage = function() {
        $scope.overlayControl(false);
        document.getElementById("editMsg").style.display = "none";
        $scope.msg.txt = "";
      };

      $scope.deleteCelestial = function(c, confirmed) {
        if (c) {
          $scope.deleting = c;
        } else if (confirmed){
          c = $scope.deleting;
        }

        // Binary celestials have a | between numbers
        var binary = c.search("\\|") !== -1;

        bngApi.engineLua("extensions.celestialController.requestDeletion('" + c + "', " + binary + ", " + confirmed + ")", (response) => {
          var state = response[0];
          if (state === "Warning" || state === "Failure") {
            $scope.showMessage(response[1], state, "Continue", "Cancel", state === "Failure");
          } else if (state === "Success") {
            $scope.deleting = "";
            $scope.$apply(function() {
              var celestialsRemoved = response[1];
              var parentsRemoved = response[2];

              for (var i = 0; i < celestialsRemoved.length; i++) {
                // This is inefficient - O(n + m)
                $scope.celestialsInWorld = $scope.celestialsInWorld.filter(celestial => celestial !== celestialsRemoved[i]);
              }

              if (parentsRemoved.length > 0) {
                var orbitingBody = $scope.path.settings.orbitingBody;
                
                for (var i = 0; i < parentsRemoved.length; i++) {
                  var parent = parentsRemoved[i];
                  // This is inefficient - O(n + m)
                  $scope.availableParentCelestials = $scope.availableParentCelestials.filter(celestial => celestial !== parent);

                  if (orbitingBody === parent){
                    //Reset orbiting body as it does not exist anymore
                    $scope.path.settings.orbitingBody = "";
                  }
                }
              }
            });

            // Remove the confirmation message
            if (confirmed) $scope.resetMessage();
          }
        });
      };

      $scope.highlightCelestial = function(c) {
        // Binary celestials have a | between numbers
        var binary = c.search("\\|") !== -1;

        var button = document.getElementById(c);

        if (button.style.backgroundColor === "") {
          button.style.backgroundColor = "#ffff00";
          bngApi.engineLua("extensions.celestialController.requestHighlightUpdate('"+ c + "', " + binary + ", true)");
        } else {
          button.style.backgroundColor = "";
          bngApi.engineLua("extensions.celestialController.requestHighlightUpdate('"+ c + "', " + binary + ", false)");
        }
      };

      $scope.changePreview = function() {
        var image = document.getElementById("model");
        image.src = "modules/apps/CelestialCreator/" + $scope.celestial.value + ".png";
      };

      $scope.fixedDynamicFields = function() {
        var objType = $scope.objectType.value;
        var isStatic = objType === "static";
        // Hide or show the path field
        if (objType === "fixedDynamic") {
          document.getElementById("path").style.display = "";
        } else if (isStatic) {
          document.getElementById("path").style.display = "none";
          // Reset to default value
          $scope.path.type.value = "";
        }

        $scope.pathFields();

        if (isStatic) {
          // Reset the path to default. Must happen after pathFields() to allow it to reset all fields first
          $scope.path.type.value = $scope.path.type.options[0].val;
        }
      };

      $scope.classFields = function() {
        var celestialClass = $scope.class.value;
        var fields = ["orbitRadius", "frequency"];

        if (celestialClass === "binary") {
          // Remove the bottom border
          document.getElementById("type").className = "top leftRight";

          // Implicitly circular and fixedDynamic
          document.getElementById("path").style.display = "none";
          $scope.path.type.value = "";
          $scope.objectType.value = $scope.objectType.options[1].val;
          $scope.pathFields();

          for (var i = 0; i < fields.length; i++) {
            var elem = document.getElementById(fields[i]);
            elem.style.display = "";

            if (i == fields.length-1) {
              elem.className = "bottom";
            }

            elem.className += " leftRight";
          }
        } else {
          // Add the bottom border
          document.getElementById("type").className += " bottom";

          for (var i = 0; i < fields.length; i++) {
            document.getElementById(fields[i]).style.display = "none";
          }

          // Set to static to begin with and default path (should it change)
          $scope.objectType.value = $scope.objectType.options[0].val;
          $scope.path.type.value = $scope.path.type.options[0].val;
        }
      }

      $scope.pathFields = function() {
        var pathType = $scope.path.type.value;

        var allFields = ["orbitingBody", "orbitRadius", "frequency", "length", "axis"];
        var circular  = ["orbitingBody", "orbitRadius", "frequency"];
        var linear    = ["frequency", "length", "axis"];
        var showFields = [];

        var circularLastRow = circular[circular.length-1];
        var linearLastRow = linear[linear.length-1];

        if (pathType === "circular") {
          // This isn't the last row anymore so remove bottom border
          document.getElementById(linearLastRow).className = "leftRight";
          // Last row will have bottom border
          document.getElementById(circularLastRow).className += " bottom";

          showFields = circular;
        } else if (pathType === "linear") {
          document.getElementById(circularLastRow).className = "leftRight";
          document.getElementById(linearLastRow).className += " bottom";

          showFields = linear;
        }

        for (var i = 0; i < allFields.length; i++){
          var field = allFields[i];
          var hide = true;

          for (var j = 0; j < showFields.length; j++) {
            if (field === showFields[j]) {
              document.getElementById(field).style.display = "";
              hide = false;

              if (field === "frequency") {
                // Linear requires frequency, where as other paths have it as optional
                document.getElementById("frequency_req").style.display = pathType === "linear" ? "none" : "";
              }

              break;
            }
          }

          // Hide if not in the show fields table
          if (hide) {
            document.getElementById(field).style.display = "none";
          }
        }
      };

      /**
      * Changes the user's selection if that celestial is not in the options anymore.
      */
      $scope.updateChosenCelestial = function() {
        var current = $scope.celestial.value;
        var options = $scope.celestial.options;

        var forceAssigned = "";

        for (var i = 0; i < options.length; i++) {
          var celestial = options[i].val;

          if (celestial === current) {
            // Object exists in the array, so return
            return;
          } else if (i === 0){
            forceAssigned = celestial;
          }
        }

        // Force assign a new celestial
        $scope.celestial.value = forceAssigned;
        $scope.changePreview();
      };

      /**
      *Changes the celestials allowed to be selected
      */
      $scope.changeAllowedCelestials = function() {
        /*
        Restrictions:
        -Binary:
          -Stars or black holes ONLY
        -Circular Pathed:
          -Cannot be a black hole
          -Cannot be a star if the orbiting body is a star
        -<Other>:
          -All celestials
        */

        // Don't change original array
        var options = JSON.parse(JSON.stringify(Options.celestials));

        var celestial = $scope.celestial.val;
        var orbitingBody = $scope.path.settings.orbitingBody;

        if ($scope.class.value === "binary") {
          // Only stars and black holes
          delete options.planets;
        // Need to make sure it is fixedDynamic since changing to static keeps the path for user convienience
        } else if ($scope.path.type.value === "circular" && $scope.objectType.value === "fixedDynamic") {
          var other = options.other;
          // Cannot be a black hole
          for (var i = 0; i < other.length; i++) {
            var element = other[i];

            if (element.val === "blackhole") {
              other.splice(i, 1);
              break;
            }
          }

          var stars = options.stars;
          // Prevent stars orbiting stars that are not binaries
          for (var i = 0; i < stars.length; i++) {
            if (orbitingBody.search(stars[i].val) !== -1) {
              delete options.stars;
            }
          }
        }

        console.log(options);

        options = Object.keys(options).reduce(
          function (result, key) {
            return result.concat(options[key]);
          },
          []
        );

        $scope.celestial.options = options;
        // Check for any incompatible choices
        $scope.updateChosenCelestial();
      };
    },
    controllerAs: "control"
  };
}]);
